Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeEvent = exports.normalizeUrl = void 0;
var electron_1 = require("electron");
/** Application base path used for URL normalization. */
var APP_PATH = electron_1.app.getAppPath().replace(/\\/g, '/');
/**
 * Normalizes URLs in exceptions and stacktraces so Sentry can fingerprint
 * across platforms.
 *
 * @param url The URL to be normalized.
 * @param base (optional) The application base path.
 * @returns The normalized URL.
 */
function normalizeUrl(url, base) {
    if (base === void 0) { base = APP_PATH; }
    // Escape RegExp special characters
    var escapedBase = base.replace(/[|\\{}()[\]^$+*?.]/g, '\\$&');
    var newUrl = url;
    try {
        newUrl = decodeURI(url);
    }
    catch (_Oo) {
        // Sometime this breaks
    }
    return newUrl
        .replace(/\\/g, '/')
        .replace(/webpack:\/?/g, '') // Remove intermediate base path
        .replace(new RegExp("(file://)?/*" + escapedBase + "/*", 'ig'), 'app:///');
}
exports.normalizeUrl = normalizeUrl;
/**
 * Returns a reference to the exception stack trace in the given event.
 * @param event An event potentially containing stack traces.
 */
function getStacktrace(event) {
    var stacktrace = event.stacktrace, exception = event.exception;
    // Try the main event stack trace first
    if (stacktrace) {
        return stacktrace;
    }
    if (exception) {
        // Raven Node adheres to the Event interface
        // @ts-ignore: need to be able to index exception
        if (exception[0]) {
            // @ts-ignore: need to be able to index exception
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            return exception[0].stacktrace;
        }
        // Raven JS uses the full values interface, which has been removed
        var raven = exception;
        if (raven.values && raven.values[0]) {
            return raven.values[0].stacktrace;
        }
    }
    return undefined;
}
/**
 * Normalizes all URLs in an event. See {@link normalizeUrl} for more
 * information. Mutates the passed in event.
 *
 * @param event The event to normalize.
 */
function normalizeEvent(event) {
    // Retrieve stack traces and normalize their URLs. Without this, grouping
    // would not work due to user folders in file names.
    var stacktrace = getStacktrace(event);
    if (stacktrace && stacktrace.frames) {
        stacktrace.frames.forEach(function (frame) {
            if (frame.filename) {
                frame.filename = normalizeUrl(frame.filename);
            }
        });
    }
    var _a = event.request, request = _a === void 0 ? {} : _a;
    if (request.url) {
        request.url = normalizeUrl(request.url);
    }
    // The user agent is parsed by Sentry and would overwrite certain context
    // information, which we don't want. Generally remove it, since we know that
    // we are browsing with Chrome.
    if (request.headers) {
        delete request.headers['User-Agent'];
    }
    // The Node SDK currently adds a default tag for server_name, which contains
    // the machine name of the computer running Electron. This is not useful
    // information in this case.
    var _b = event.tags, tags = _b === void 0 ? {} : _b;
    delete tags.server_name;
    return event;
}
exports.normalizeEvent = normalizeEvent;
//# sourceMappingURL=normalize.js.map